//    Copyright (C) 2004 Michael Mahr

//    This file is part of "lesschess".
//
//    "lesschess" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "lesschess" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "lesschess"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __chessboard__
#define __chessboard__

#include "defs.h"
#include "chesspiece.h"

//This class contains all the game-logic - if you want to add some game logic, then the .cpp to this class is the right place.

//everything that has to do with graphics is contained in the derived class

//the template parameter for instanciating this class must be inherited from "chesspiece"
//the reason for using templates here is that the chessboard must be able to instanciate chesspieces on its own.

class ChessboardException : public Exception
{
  public:
    ChessboardException():Exception(){};
    ChessboardException(std::string _message): Exception(_message){};
};

template <class T>
class Chessboard
{
private:
  Chesspiece* chesspieces[32];
  int fallen[2]; //counts the number of fallen pieces on each side.
  bool whiteturn;

  Chesspiece* whatison(int _posx, int _posy);

public:
  Chessboard ();

  bool squareisblack(int _posx,int _posy);

  bool moveallowed (int _posx1, int _posy1, int _posx2, int _posy2);

  void removepiece(Chesspiece* piece);

  void trymove (int _posx1, int _posy1, int _posx2, int _posy2);

  bool iswhiteturn();

  void drawpieces ();

  virtual void draw();

  bool posoffboard(int _posx, int _posy);

};

//-------------------------------
//-------------------------------
//-------------------------------


	template <class T>
	Chessboard<T>::Chessboard ()
	{
		//alle 32 Figuren anlegen

		//weier Kader
		chesspieces[0] = new T (true,2,0,0);
		chesspieces[1] = new T (true,1,1,0);
		chesspieces[2] = new T (true,3,2,0);
		chesspieces[3] = new T (true,4,3,0);
		chesspieces[4] = new T (true,5,4,0);
		chesspieces[5] = new T (true,3,5,0);
		chesspieces[6] = new T (true,1,6,0);
		chesspieces[7] = new T (true,2,7,0);

		//weie Bauern
		chesspieces[8] = new T (true,0,0,1);
		chesspieces[9] = new T (true,0,1,1);
		chesspieces[10] = new T (true,0,2,1);
		chesspieces[11] = new T (true,0,3,1);
		chesspieces[12] = new T (true,0,4,1);
		chesspieces[13] = new T (true,0,5,1);
		chesspieces[14] = new T (true,0,6,1);
		chesspieces[15] = new T (true,0,7,1);

		//schwarze Bauern
		chesspieces[16] = new T (false,0,0,6);
		chesspieces[17] = new T (false,0,1,6);
		chesspieces[18] = new T (false,0,2,6);
		chesspieces[19] = new T (false,0,3,6);
		chesspieces[20] = new T (false,0,4,6);
		chesspieces[21] = new T (false,0,5,6);
		chesspieces[22] = new T (false,0,6,6);
		chesspieces[23] = new T (false,0,7,6);

		//schwarzer Kader
		chesspieces[24] = new T (false,2,0,7);
		chesspieces[25] = new T (false,1,1,7);
		chesspieces[26] = new T (false,3,2,7);
		chesspieces[27] = new T (false,4,3,7);
		chesspieces[28] = new T (false,5,4,7);
		chesspieces[29] = new T (false,3,5,7);
		chesspieces[30] = new T (false,1,6,7);
		chesspieces[31] = new T (false,2,7,7);
	
		fallen[0]=0;
		fallen[1]=0;

		whiteturn = true;
	}

	template <class T>
	bool Chessboard<T>::squareisblack(int _posx,int _posy)
	{
		if (posoffboard(_posx,_posy))
		{
			throw ChessboardException("The position is not on the board.");
		}

		return ((_posx+_posy)%2);
	}


        template <class T>
	bool Chessboard<T>::moveallowed (int _posx1, int _posy1, int _posx2, int _posy2)
	{
		if ((_posx1==_posx2) && (_posy1==_posy2))
		{
			//same pos
			return false;
		}

		if ((posoffboard(_posx1,_posy1)) || (posoffboard(_posx1,_posy1)))
		{
			throw ChessboardException("The specified position is off the board.");
		}

		Chesspiece * startpiece = whatison(_posx1,_posy1);
		Chesspiece * endpiece = whatison(_posx2,_posy2);


		if (startpiece ==0 )
		{
			//startposition ist leer
			return false;
		}
		else
		{
			//ist die Farbe dran?
			if (startpiece->getwhite() != whiteturn)
			{
				return false;
			}

			if (endpiece !=0 ) //zug auf gleichfarbigen
			{
				//rochade kann hier eingebaut werden
				if (endpiece->getwhite()==startpiece->getwhite())
				{return false;}
			}


			if ((endpiece==0) || (startpiece->getwhite() != endpiece->getwhite()))
			{
				//zielposition ist leer
				switch (startpiece->getvalue())
				{
					case 0: //pawn
						if (endpiece==0) //endpos ist leer
						{
							if (startpiece->getwhite()== true)
							{
								if (((_posy1+1)==_posy2) && (_posx1==_posx2)) {return true;}
							}
							else
							{
								if (((_posy1-1)==_posy2) && (_posx1==_posx2)) {return true;}
							}
							//sonderfall des ersten Zuges eines Bauern
							if (startpiece->getwhite()== true)
							{
								if (((_posy1+2)==_posy2) && (_posx1==_posx2)&&_posy1==1) {return true;}
							}
							else
							{
								if (((_posy1-2)==_posy2) && (_posx1==_posx2)&&_posy1==6) {return true;}
							}
						}
						else //endpos ist besetzt
						{
							if (startpiece->getwhite()== true)
							{
								if (((_posy1+1)==_posy2) && (_posx1==_posx2+1)) {return true;}
								if (((_posy1+1)==_posy2) && (_posx1==_posx2-1)) {return true;}
							}
							else
							{
								if (((_posy1-1)==_posy2) && (_posx1==_posx2+1)) {return true;}
								if (((_posy1-1)==_posy2) && (_posx1==_posx2-1)) {return true;}
							}

						}
						break;

					case 1: //knight
						if (((_posy1-_posy2)==2)||((_posy2-_posy1)==2))
						{
							if (((_posx1-_posx2)==1)||((_posx2-_posx1)==1))
							{
								return true;
							}
						}

						if (((_posx1-_posx2)==2)||((_posx2-_posx1)==2))
						{
							if (((_posy1-_posy2)==1)||((_posy2-_posy1)==1))
							{
								return true;
							}
						}

						break;


					case 2: //rook
						if (_posx1==_posx2)
						{
							if (_posy2 > _posy1)
							{
								for (int i=_posy1+1; i < _posy2;i++)
								{
									if (whatison(_posx1,i)!=0)
									{
										return false;
									}
								}
								return true;
							}
							else if (_posy1 > _posy2)
							{
								for (int i=_posy2+1; i < _posy1;i++)
								{
									if (whatison(_posx2,i)!=0)
									{
										return false;
									}
								}
								return true;
							}
						}
						if (_posy1==_posy2)
						{

							if (_posx2 > _posx1)
							{
								for (int i=_posx1+1; i < _posx2;i++)
								{
									if (whatison(i,_posy1)!=0)
									{
										return false;
									}
								}
								return true;
							}
							else if (_posx1 > _posx2)
							{
								for (int i=_posx2+1; i < _posx1;i++)
								{
									if (whatison(i,_posy2)!=0)
									{
										return false;
									}
								}
								return true;
							}

						}
						break;

					case 3: //bishop
						if ((_posx1 - _posx2) == (_posy1 - _posy2))
						{
							if (_posx1 > _posx2)
							{
								for (int i=1; i < _posx1-_posx2;i++)
								{
									if (whatison(_posx1-i,_posy1-i)!=0)
									{
										return false;
									}
								}
								return true;
							}
							else if (_posx2 > _posx1)
							{
								for (int i=1; i < _posx2-_posx1;i++)
								{
									if (whatison(_posx1+i,_posy1+i)!=0)
									{
										return false;
									}
								}
								return true;
							}
						}
						if ((_posx1 - _posx2) == -(_posy1 - _posy2))
						{
							if (_posx1 > _posx2)
							{
								for (int i=1; i < _posx1-_posx2;i++)
								{
									if (whatison(_posx1-i,_posy1+i)!=0)
									{
										return false;
									}
								}
								return true;
							}
							else if (_posx2 > _posx1)
							{
								for (int i=1; i < _posx2-_posx1;i++)
								{
									if (whatison(_posx1+i,_posy1-i)!=0)
									{
										return false;
									}
								}
								return true;
							}
						}
						break;

					case 4: //queen besteht aus rook und bishop

						//rookteil
						if (_posx1==_posx2)
						{
							if (_posy2 > _posy1)
							{
								for (int i=_posy1+1; i < _posy2;i++)
								{
									if (whatison(_posx1,i)!=0)
									{
										return false;
									}
								}
								return true;
							}
							else if (_posy1 > _posy2)
							{
								for (int i=_posy2+1; i < _posy1;i++)
								{
									if (whatison(_posx2,i)!=0)
									{
										return false;
									}
								}
								return true;
							}
						}
						if (_posy1==_posy2)
						{

							if (_posx2 > _posx1)
							{
								for (int i=_posx1+1; i < _posx2;i++)
								{
									if (whatison(i,_posy1)!=0)
									{
										return false;
									}
								}
								return true;
							}
							else if (_posx1 > _posx2)
							{
								for (int i=_posx2+1; i < _posx1;i++)
								{
									if (whatison(i,_posy2)!=0)
									{
										return false;
									}
								}
								return true;
							}

						}

						//bishopteil
						if ((_posx1 - _posx2) == (_posy1 - _posy2))
						{
							if (_posx1 > _posx2)
							{
								for (int i=1; i < _posx1-_posx2;i++)
								{
									if (whatison(_posx1-i,_posy1-i)!=0)
									{
										return false;
									}
								}
								return true;
							}
							else if (_posx2 > _posx1)
							{
								for (int i=1; i < _posx2-_posx1;i++)
								{
									if (whatison(_posx1+i,_posy1+i)!=0)
									{
										return false;
									}
								}
								return true;
							}
						}
						if ((_posx1 - _posx2) == -(_posy1 - _posy2))
						{
							if (_posx1 > _posx2)
							{
								for (int i=1; i < _posx1-_posx2;i++)
								{
									if (whatison(_posx1-i,_posy1+i)!=0)
									{
										return false;
									}
								}
								return true;
							}
							else if (_posx2 > _posx1)
							{
								for (int i=1; i < _posx2-_posx1;i++)
								{
									if (whatison(_posx1+i,_posy1-i)!=0)
									{
										return false;
									}
								}
								return true;
							}
						}

						break;

					case 5: //king
						if (((_posx1+2)> _posx2)&&((_posx1-2)< _posx2)&&((_posy1+2)> _posy2)&&((_posy1-2)< _posy2)) {return true;}
						break;
				}



			}


		}

		return false;
	}


	template <class T>
	void Chessboard<T>::removepiece(Chesspiece* piece)
	{

		if (piece->getwhite()) // wo die hinen hinkommen
		{
			if ((fallen[1]< 8))
			{
				piece->hitmove(fallen[1] % 8, -2); //todo
			}
			else
			{
				piece->hitmove(fallen[1] % 8, -3); //todo
			}
			fallen[1]++;
		}
		else
		{
			if ((fallen[0]< 8))
			{
				piece->hitmove(fallen[0] % 8, 9); //todo
			}
			else
			{
				piece->hitmove(fallen[0] % 8, 10); //todo
			}
			fallen[0]++;
		}


	}

	template <class T>
	void Chessboard<T>::trymove (int _posx1, int _posy1, int _posx2, int _posy2)
	{
		if (moveallowed(_posx1, _posy1, _posx2, _posy2))
		{
			//wenn die Bewegung in Ordnung geht

			Chesspiece* endpiece = whatison(_posx2,_posy2);

			if (endpiece==0)
			{
				//wenn das Zielfeld leer ist
			}
			else
			{
				//wenn das Zielfeld besetzt ist;
				removepiece(endpiece);
			}

			//in beiden
			Chesspiece* startpiece = whatison(_posx1,_posy1);

			startpiece->moveto(_posx2,_posy2);

			//ein Zug wird verbraucht
			whiteturn = (!whiteturn);

		}
		else
		{
			return;
		}
	}

	template <class T>
	bool Chessboard<T>::iswhiteturn()
	{
		return whiteturn;
	}

	template <class T>
	void Chessboard<T>::drawpieces ()
	{	
		for (int x = 0; x < 32; x++)
		{
			chesspieces[x]->draw();
		}
	}

	template <class T>
	void Chessboard<T>::draw()
	{
		drawpieces();
	}

	template <class T>
	bool Chessboard<T>::posoffboard(int _posx, int _posy)
	{
		if ((_posx < 0) || (_posx > 7)) {return true;}
		if ((_posy < 0) || (_posy > 7)) {return true;}
		return false;
	}

	template <class T>
	Chesspiece* Chessboard<T>::whatison(int _posx, int _posy)
	{
		for (int x =0; x < 32; x++)
		{
			if ((chesspieces[x]->getx()==_posx) && (chesspieces[x]->gety()==_posy) )
			{
				return chesspieces[x];
			}
		}
		return 0;
	}



#endif
