//    Copyright (C) 2004 Michael Mahr

//    This file is part of "PS2L_UtilClasses".
//
//    "PS2L_UtilClasses" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "PS2L_UtilClasses" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "PS2L_UtilClasses"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "ds2pad.h"


struct PS2PadMode
{
  int offset;
  int lock;
};

struct DS2PadActuators
{
  int len;
  unsigned char data[32];
};

DS2Pad::DS2Pad(std::string _devfilename)
{
  fd = open(_devfilename.c_str(),O_RDWR);
  
  if (fd == -1)
  {
    //read/write is necessary for ioctls to work
    std::cout << "could not open \"" << _devfilename.c_str() << "\" for read/write access" <<  std::endl;
    exit(0);
  }
  
  //set the pad to its initial state (red light is left untouched, everything else is disabled)
  pm = true;
  lock = true;
  act = true;
  
  lockmode(false); //set lock and store analogmode
  pressuremode(false);
  actuatormode(false);  

}


void DS2Pad::update()
{ 

  //not a nice thing, however you could not be sure that pressures really are ON otherwise
  if( pressuremode() && analogmode() )
  {
      waitready();
  
      //switch on pressures
      ioctl(fd, 0x20007001);
  }

  //the same with actuators... - otherwise get turned off during a modechange
  if(actuatormode())
  {
    waitready();
  
    struct DS2PadActuators pact;
    memset(pact.data, -1, sizeof(pact.data));    
    pact.len = 6;
    pact.data[0] = 0;
    pact.data[1] = 1;
    ioctl(fd, 0x80247009, &pact);

    waitready();

    struct DS2PadActuators pint;
    memset(pint.data, -1, sizeof(pint.data));
    pint.len = 6;
    pint.data[0] = smallactuator?1:0;
    pint.data[1] = bigactuator;
    ioctl(fd, 0x8024700a, &pint);
    
  }

  waitready();

  unsigned char padbuf[32];
  read(fd, padbuf, 32);

  pressed[DS2PadButton(sel)]   = !(padbuf[2] & 1); 
  pressed[DS2PadButton(l3)]    = !(padbuf[2] & 2); 
  pressed[DS2PadButton(r3)]    = !(padbuf[2] & 4); 
  pressed[DS2PadButton(start)] = !(padbuf[2] & 8); 

  pressed[DS2PadButton(up)]    = !(padbuf[2] & 16); 
  pressed[DS2PadButton(right)] = !(padbuf[2] & 32); 
  pressed[DS2PadButton(down)]  = !(padbuf[2] & 64); 
  pressed[DS2PadButton(left)]  = !(padbuf[2] & 128); 

  pressed[DS2PadButton(l2)] = !(padbuf[3] & 1); 
  pressed[DS2PadButton(r2)] = !(padbuf[3] & 2); 
  pressed[DS2PadButton(l1)] = !(padbuf[3] & 4); 
  pressed[DS2PadButton(r1)] = !(padbuf[3] & 8); 

  pressed[DS2PadButton(triangle)] = !(padbuf[3] & 16); 
  pressed[DS2PadButton(circle)]   = !(padbuf[3] & 32); 
  pressed[DS2PadButton(cross)]    = !(padbuf[3] & 64); 
  pressed[DS2PadButton(square)]   = !(padbuf[3] & 128); 

  if (pressuremode() && analogmode() )
  {
    //start, select, l3 and r3 do not have pressures
    //pressed and pressure are not stored in the same order in the devfile
  
    pressure[DS2PadButton(up)]    = padbuf[10]; 
    pressure[DS2PadButton(right)] = padbuf[8]; 
    pressure[DS2PadButton(down)]  = padbuf[11];  
    pressure[DS2PadButton(left)]  = padbuf[9];  

    pressure[DS2PadButton(l2)] = padbuf[18]; 
    pressure[DS2PadButton(r2)] = padbuf[19];  
    pressure[DS2PadButton(l1)] = padbuf[16];  
    pressure[DS2PadButton(r1)] = padbuf[17];  

    pressure[DS2PadButton(triangle)] = padbuf[12]; 
    pressure[DS2PadButton(circle)]   = padbuf[13];  
    pressure[DS2PadButton(cross)]    = padbuf[14];  
    pressure[DS2PadButton(square)]   = padbuf[15];  
  }

  //--stickpositions
  rightstickx = padbuf[4];
  rightsticky = padbuf[5];

  leftstickx = padbuf[6];
  leftsticky = padbuf[7];
  

}

void DS2Pad::pressuremode(bool _pm)
{
  waitready();

  if (pm && !(_pm))
    ioctl(fd, 0x20007002);  //disable pressures

  pm = _pm;
}
bool DS2Pad::pressuremode()
{
  return pm;
}


bool DS2Pad::analogmode()
{
  waitready();

  int retval;
  ioctl(fd, 0x40047000, &retval); //check analogmode with whether pressures are avaliable
  return (retval == 1);
}


void DS2Pad::analogmode_internal(bool _am)
{
  waitready();

  struct PS2PadMode pm;
  pm.offset = _am?1:0; //use analogmode
  pm.lock = lock?3:2;  
  ioctl(fd, 0x80087008, &pm);
}

void DS2Pad::analogmode(bool _am)
{
  if (lockmode())
    am = _am;  //store analogmode for error-recovery if lockmode is enabled
    
  analogmode_internal(_am);
}


bool DS2Pad::lockmode()
{
  return lock;
}


void DS2Pad::lockmode_internal(bool _lock)
{
  waitready();

  lock = _lock;

  struct PS2PadMode pm;
  pm.offset = analogmode();  //use current analogmode
  pm.lock = lock?3:2;  
  ioctl(fd, 0x80087008, &pm);
}

void DS2Pad::lockmode(bool _lock) //the external lockmode
{
  am = analogmode(); //store analogmode for error-recovery
  lockmode_internal(_lock);
}

bool DS2Pad::actuatormode()
{
  return act;
}

void DS2Pad::actuatormode(bool _act)
{
  waitready();
  
  if (act && !(_act) ) //if being disabled
  {
    struct DS2PadActuators pact;
    memset(pact.data, -1, sizeof(pact.data));

    pact.len = 6;
    ioctl(fd, 0x80247009, &pact);
  }
  
  act = _act;
}

int DS2Pad::checkrstat()
{
  int retval;
  ioctl(fd, 0x40047003, &retval);
  return retval;
}

int DS2Pad::checkstat()
{
  int retval;
  ioctl(fd, 0x40047004, &retval);
  return retval;
}

void DS2Pad::waitready()
{
  //waits until the controller becomes ready or restores state after errors

  unsigned int enumber = 100;
  bool errorhappened = false;
  
  while ((enumber = checkstat()) != 1) //wait until the controller is ready
  {
    if (enumber == 3) errorhappened = true;
  }

  if (errorhappened)
  {
    //restore analogmode only if pad was locked - otherwise the mode is left off
    if (lock)
    {
      analogmode_internal(am);
    }

    //restore lockmode
    lock = (!lock);
    lockmode_internal(!lock);

    //restore pressuremode
    pm = (!pm);
    pressuremode(!pm);

    //restore actuatormode
    act = (!act);
    actuatormode(!act);
    
  }
}

DS2Pad::~DS2Pad()
{
  lockmode(false);
  pressuremode(false);
  actuatormode(false);
}
