//    Copyright (C) 2004 Michael Mahr

//    This file is part of "PS2L_UtilClasses".
//
//    "PS2L_UtilClasses" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "PS2L_UtilClasses" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "PS2L_UtilClasses"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __ds2pad__
#define __ds2pad__

#include <unistd.h>
#include <fcntl.h>
#include <string>
#include <sys/ioctl.h>
#include <iostream>

//note: this class only works properly with a DualShock2 pad
//when created the class resets the pad to: unlocked, nopressure. The red light is left untouched.
//after destroying the class the pad will be set to the same state

//only one object per pad!

//and... you should lock the pad using lockmode(true)

//for analogsticks analogmode is required
//for buttonpressures pressuremode and analogmode is required

enum DS2PadButton
{
  sel = 0,
  l3 = 1,   //button available on the PS2-remote control SCHP-10420
  r3 = 2,   //button available on the PS2-remote control SCHP-10420
  start = 3,
 
  up = 4,
  right = 5,
  down = 6,
  left = 7,

  l2 = 8,
  r2 = 9,
  l1 = 10, 
  r1 = 11,
  
  triangle = 12,
  circle = 13,
  cross = 14,
  square = 15,
};


class DS2Pad
{
public:

  //read the data from here
  bool pressed[16];
  unsigned char pressure[16]; // use "analogmode(true)" and "pressuremode(true)" to get pressuredata
  unsigned char leftstickx; // use "analogmode(true)" first to read the stick-data
  unsigned char leftsticky; 
  unsigned char rightstickx; 
  unsigned char rightsticky; 

  //write the data in here (but use "actuatormode(true)" first)
  unsigned char bigactuator;
  bool smallactuator;

  //the basics
  DS2Pad(std::string _devfilename);
  void update();  // call this once before reading the values!
  ~DS2Pad();

  bool analogmode(); //returns true if "red light" is on
  void analogmode(bool); //sets the red light

  bool pressuremode(); //returns if pressure should be measured
  void pressuremode(bool); // supply true if you also want to read button-pressures

  //note!: analogmode() _and_ pressuremode() have to be _both_ _enabled_ so you can really read pressures!
  //this is a hardware-limitation
  
  bool lockmode(); //returns whether the analogmode of the pad is locked
  void lockmode(bool); //set the locking status
  
  bool actuatormode(); //returs whether the actuators are enabled
  void actuatormode(bool); //enables or disables the actuators


  //the following functions are usually not directly invoked - for debugging
  int checkrstat();
  int checkstat();

private:
  int fd;  //the filedescriptor of the file
  bool pm; //whether pressuremode is enabled
  bool lock; //whether locking is enabled
  bool act; //whether actuators are enabled
  
  bool am; //stores the analogmode for error-recovery
  
  void waitready();  //blocks execution until the controll is ready

  void lockmode_internal(bool); //this function differs in that am is not set
  void analogmode_internal(bool); //this function differs in that am is not set  
};

#endif

