//    Copyright (C) 2004 Michael Mahr

//    This file is part of "lesschess".
//
//    "lesschess" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "lesschess" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "lesschess"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "game.h"

Game::Game()
{
  xrotation.init(-150);
  yrotation.init(-140);
  distance.init(15);
  
  xrotation.value=700;
  yrotation.value=-1080;
  distance.value=200;
}

Game::~Game()
{
 
}

void Game::relmove(int x, int y)
{
  ensure360();

  if ((xrotation.target >= 45) && (xrotation.target < 135))
  {
    board.relcursormove(y,x);
  }
  else if ((xrotation.target >= 135) && (xrotation.target < 225))
  {
    board.relcursormove(x,-y);
  }
  else if ((xrotation.target >= 225) && (xrotation.target < 315))
  {
    board.relcursormove(-y,-x);
  }
  else
  {
    board.relcursormove(-x,y);
  }
}

void Game::ensure360()
{
  //the current cameraparams must stay in sync with the normal cameraparams

  while (xrotation.target < 0)
  {
    xrotation.target += 360;
    xrotation.value += 360;
  }
  while (xrotation.target > 360)
  {
    xrotation.target -= 360;
    xrotation.value -= 360;
  }
  while (yrotation.target < 0)
  {
    yrotation.target += 360;
    yrotation.value += 360;
  }
  while (yrotation.target > 360)
  {
    yrotation.target -= 360;
    yrotation.value -= 360;
  }

}

bool Game::draw(unsigned int msecs, unsigned char * buttons)
{

  unsigned int rotspeed = 10;
  unsigned int zoomspeed = 100;
  
  
  //ds2pad exit
  if ((buttons[DS2PadButton(start)] & 4)&&(buttons[DS2PadButton(sel)] & 4))
    return false;
  
  //ds2pad analog sticks - move camera
  float deadzone = 0.15;
  
  float rxstick = (((float)buttons[16]-127.0)/255.0);
  float rystick = (((float)buttons[17]-127.0)/255.0);
  float lxstick = (((float)buttons[18]-127.0)/255.0);
  float lystick = (((float)buttons[19]-127.0)/255.0);

  if ((rxstick < deadzone) && (rxstick > -deadzone)) rxstick = 0;
  if ((rystick < deadzone) && (rystick > -deadzone)) rystick = 0;
  if ((lxstick < deadzone) && (lxstick > -deadzone)) lxstick = 0;
  if ((lystick < deadzone) && (lystick > -deadzone)) lystick = 0;
  
  xrotation.target += (((float)msecs)/rotspeed)* rxstick;
  yrotation.target += (((float)msecs)/rotspeed)* rystick;
  xrotation.target += (((float)msecs)/rotspeed)* lxstick;
  yrotation.target += (((float)msecs)/rotspeed)* lystick;

  //ds2pad zoom via l1 l2 r1 r2
  if (buttons[DS2PadButton(l1)] & 4)
    distance.target -= ((float)msecs)/zoomspeed;
  if (buttons[DS2PadButton(r1)] & 4)
    distance.target -= ((float)msecs)/zoomspeed;
  if (buttons[DS2PadButton(l2)] & 4)
    distance.target += ((float)msecs)/zoomspeed;
  if (buttons[DS2PadButton(r2)] & 4)
    distance.target += ((float)msecs)/zoomspeed;
  
  //The user cannot zoom nearer than 9 units - this is vital for the vu0 not to crash (since no clipping is done)
  //If you are curious, remove this line and zoom really close.
  if (distance.target < 9) distance.target = 9;
  
  //ds2pad-cursor-movement
  if (buttons[DS2PadButton(up)] & 1)
    relmove(0,1);  
  if (buttons[DS2PadButton(down)] & 1)
    relmove(0,-1);  
  if (buttons[DS2PadButton(left)] & 1)
    relmove(1,0);  
  if (buttons[DS2PadButton(right)] & 1)
    relmove(-1,0);  
  
  if (buttons[DS2PadButton(cross)] & 1)
    board.activate();  
  

  //calculate camera smoothing
  xrotation.calculate(((float) msecs)/200);
  yrotation.calculate(((float) msecs)/200);
  distance.calculate(((float) msecs)/200);


  //move camera
  matrixmanager.modelviewPushTranslate(0,0, -distance.value);
  matrixmanager.modelviewPushRotate(1,0,0, -yrotation.value);
  matrixmanager.modelviewPushRotate(0,1,0, -xrotation.value);
  
  board.draw();
  
  matrixmanager.modelviewPop();
  matrixmanager.modelviewPop();
  matrixmanager.modelviewPop();
  
  return true;
}
