//    Copyright (C) 2004 Michael Mahr

//    This file is part of "lesschess".
//
//    "lesschess" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "lesschess" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "lesschess"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <iostream>
#include "defs.h"
#include "game.h"

#include "timer.h"
#include "padmanager.h"


Game * game; //a global object that contains the whole game
Timer timer; //calculates the timedifference to the last frame
Padmanager padmanager;

int sps2Descriptor; //used for working with sps2

void draw()
{ 
  sps2UScreenClear(0x80, 0x80, 0x80);
 

  //draws everything
  if (!(game->draw(timer.msDiff(),padmanager.getbuttons())))
  {
    //if the game decides to end itself
    delete game;
    exit(0);
  }

        
  sps2UScreenSwap();
}


int main (int argc, char * argv[])
{

  std::cout <<  "\"lesschess\" version 0.1.1-PS2, Copyright (C) Michael Mahr" << std::endl;
  std::cout <<  "\"lesschess\" comes with ABSOLUTELY NO WARRANTY. This is" << std::endl;
  std::cout <<  "free software, and you are welcome to redistribute it" << std::endl;
  std::cout <<  "under certain conditions; see \"Copying\" for details." << std::endl;
  std::cout << std::endl;
  std::cout << "press <start> and <select> to exit" << std::endl;
  std::cout << std::endl;
  std::cout << "loading game data..." << std::endl;
  game = new Game;
  std::cout << std::endl;

  timer.msDiff();

  sps2Descriptor = sps2Init();
  
  if (sps2Descriptor < 0)
  {
    std::cout << "sps2Init() failed" << std::endl;
    exit (0);
  }

  sps2UScreenInit(0);

  //all the perspective-and screenfit-stuff is set here
  //note: the perspectivematrix and the screenmatrix here are not 100% right,
  //  but they fit together and since no clipping is done...

  float width = 720; //I am not sure how wide the ps2sceen is exactly- its a guess
  float height = 576; //ditto
  float zdepth = 65535;  //the z-value at infinity (the z-value at the near clipping-plane is 0)

  matrixmanager.perspectiveInfinite(1.0f, width/height, 90);
  
  //the else-block usually gets executed - ignore the first block
  if ((argc == 2) && (strcmp(argv[1], "-mcesher")==0))   
  {
    float screenfitmatrix[] = {width,0,0,0,0,height,0,0,0,0,-1000,0,2048,2048,65000,1};
    matrixmanager.screenfit(screenfitmatrix);  
  }
  else
  {
    //converts from clip-space to screen-space (scales by width, height and z-depth and shifts by 2048)
    float screenfitmatrix[] = {width, 0, 0, 0, 0, height, 0, 0, 0, 0, -zdepth/2, 0, 2048, 2048, zdepth/2, 1};
    matrixmanager.screenfit(screenfitmatrix);  
  }
  
  for(;;)  //the game loop
  {
    draw();
  }
  
  sps2UScreenShutdown();
  sps2Release(sps2Descriptor);
  
  return 0;
}
