//    Copyright (C) 2004 Michael Mahr

//    This file is part of "lesschess".
//
//    "lesschess" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "lesschess" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "lesschess"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __sps2chessboard__
#define __sps2chessboard__

#include <iostream>

#include "defs.h"
#include <string.h>
#include "sps2ascmodel.h"
#include "chessboard.h"
#include "sps2uvsphere.h"

template <class T>
class SPS2Chessboard: public Chessboard <T>
{
private:

  SPS2AscModel * ringblack;
  SPS2AscModel * ringwhite;

  SPS2UVSphere * sphereblack;
  SPS2UVSphere * spherewhite;

  int cursorx;
  int cursory;

  int cursor2x;
  int cursor2y;

  bool active;

  //this function only pushes the verticles...
  void drawsquare(sps2GIFPackedRegister_t &GIFRegColor, const float &x1, const float &y1, const float &z1, const float &x2, const float &y2, const float &z2, const float &x3, const float &y3, const float &z3, const float &x4, const float &y4, const float &z4);

  
public:
  SPS2Chessboard();
  virtual ~SPS2Chessboard();

  void draw();

  void relcursormove (int _relx,int _rely);

  void activate();

};

//----------------
//----------------
//----------------

template <class T>
SPS2Chessboard<T>::SPS2Chessboard():Chessboard<T>()
{
  cursorx = 5;
  cursory = 5;

  active = false;

  cursor2x = cursorx;
  cursor2y = cursory;

  float modelscale[] = {0.1,0.1,0.1};
  float modelrotation[] = {1,0,0,-90};
  float blackcolor[] = {0,0,0};
  float whitecolor[] = {0.6,0.6,0.6};
  float lightcolor[] = {0.5,0.5,0.5};
  float lightorigin[] = {-1,0.5,-1};
  
  ringblack = new SPS2AscModel("./ring.asc", modelscale, modelrotation, blackcolor, lightcolor, lightorigin);
  ringwhite = new SPS2AscModel("./ring.asc", modelscale, modelrotation, whitecolor, lightcolor, lightorigin);  
  
  spherewhite = new SPS2UVSphere(4,8,0.1666, 200, 200, 200, 0x80);
  sphereblack = new SPS2UVSphere(4,8,0.1666, 20, 20, 20, 0x80);    
}


template <class T>
void SPS2Chessboard<T>::drawsquare(sps2GIFPackedRegister_t &GIFRegColor, const float &x1, const float &y1, const float &z1, const float &x2, const float &y2, const float &z2, const float &x3, const float &y3, const float &z3, const float &x4, const float &y4, const float &z4)
{
  float top1[] __attribute__((aligned(16))) = {x1, y1, z1, 1.0f, x2, y2, z2, 1.0f, x3, y3, z3, 1.0f};
  float top2[] __attribute__((aligned(16))) = {x3, y3, z3, 1.0f, x4, y4, z4, 1.0f, x1, y1, z1, 1.0f};
  unsigned int top1d[12] __attribute__((aligned(16)));
  unsigned int top2d[12] __attribute__((aligned(16)));
  matrixmanager.facetransformi4(top1d, top1);
  matrixmanager.facetransformi4(top2d, top2);

  sps2GIFPackedRegister_t GIFReg;
  GIFReg.i128 = 0;
				
  //draw first face
  HACK_DPUT_EE_GIF_FIFO(GIFRegColor.i128); //set color
  memcpy(&GIFReg.XYZ2.i128, &top1d[0], 12);
  HACK_DPUT_EE_GIF_FIFO(GIFReg.i128);
  
  HACK_DPUT_EE_GIF_FIFO(GIFRegColor.i128); //set color
  memcpy(&GIFReg.XYZ2.i128, &top1d[4], 12);
  HACK_DPUT_EE_GIF_FIFO(GIFReg.i128);
  
  HACK_DPUT_EE_GIF_FIFO(GIFRegColor.i128); //set color
  memcpy(&GIFReg.XYZ2.i128, &top1d[8], 12);
  HACK_DPUT_EE_GIF_FIFO(GIFReg.i128);
  
  //draw second face
  HACK_DPUT_EE_GIF_FIFO(GIFRegColor.i128); //set color
  memcpy(&GIFReg.XYZ2.i128, &top2d[0], 12);
  HACK_DPUT_EE_GIF_FIFO(GIFReg.i128);
  
  HACK_DPUT_EE_GIF_FIFO(GIFRegColor.i128); //set color
  memcpy(&GIFReg.XYZ2.i128, &top2d[4], 12);
  HACK_DPUT_EE_GIF_FIFO(GIFReg.i128);
  
  HACK_DPUT_EE_GIF_FIFO(GIFRegColor.i128); //set color
  memcpy(&GIFReg.XYZ2.i128, &top2d[8], 12);
  HACK_DPUT_EE_GIF_FIFO(GIFReg.i128);

  /*
  std::cout << (top1d[0]>>4) << std::endl;
  std::cout << (top1d[1]>>4) << std::endl;
  std::cout << (top1d[2]>>4) << std::endl;
  std::cout << (top1d[3]>>4) << std::endl;
  */

}

template <class T>
void SPS2Chessboard<T>::draw()
{

		//draw the cursors
                matrixmanager.modelviewPushTranslate(cursorx-3.5,0,cursory-3.5);
		if(iswhiteturn())
		  ringwhite->draw();
		else
		  ringblack->draw();
		matrixmanager.modelviewPop();


		//if the cursors are not on the same position - draw the second one too
		if ((cursor2x != cursorx) || (cursor2y != cursory))
		{
		        matrixmanager.modelviewPushTranslate(cursor2x-3.5,0,cursor2y-3.5);
         		if(iswhiteturn())
			  ringwhite->draw();
			else
			  ringblack->draw();
			matrixmanager.modelviewPop();
		}


		for (int a=0; a < 8; a ++)
		{
		        for (int b=0; b < 8; b ++)
			{	
			
				matrixmanager.loadmatrix();
			
				//calculate the number of squares to draw
				unsigned int squares = 2;
				
				if (a==0) squares++;
				if (b==0) squares++;
				if (a==7) squares++;
				if (b==7) squares++;

				sps2GIFTag_t GIFTag;

				GIFTag.i128 = 0;
				GIFTag.s.NLOOP = squares * 6;
				GIFTag.s.PRE = 1;
				GIFTag.s.EOP = 1;
				GIFTag.s.PRIM = 11;
				GIFTag.s.FLG = GIF_FLG_PACKED;
				GIFTag.s.NREG = 0;
				
				SPS2_SET_GIF_REG(GIFTag, GIFTag.s.NREG++, GIF_REG_RGBAQ);
				SPS2_SET_GIF_REG(GIFTag, GIFTag.s.NREG++, GIF_REG_XYZ2);

				HACK_DPUT_EE_GIF_FIFO(GIFTag.i128);

				sps2GIFPackedRegister_t GIFRegColor;			
				
				GIFRegColor.i128 = 0;


				if (squareisblack(a,b)) //normales Brett
				{
					GIFRegColor.RGBAQ.s.R = 20;
					GIFRegColor.RGBAQ.s.G = 20;
					GIFRegColor.RGBAQ.s.B = 20;
					GIFRegColor.RGBAQ.s.A = 0x80;
				}
				else
				{
					GIFRegColor.RGBAQ.s.R = 200;
					GIFRegColor.RGBAQ.s.G = 200;
					GIFRegColor.RGBAQ.s.B = 200;
					GIFRegColor.RGBAQ.s.A = 0x80;
				}
				
				while((*EE_GIF_STAT & (0x1F << 24))>8);
				

				drawsquare(GIFRegColor, a-3, -0.25f, b-3, a-4, -0.25f, b-3, a-4, -0.25f, b-4, a-3, -0.25f, b-4);				
				drawsquare(GIFRegColor, a-3, 0, b-4, a-4, 0, b-4, a-4, 0, b-3, a-3, 0, b-3);


				if (a==0)
				{
					drawsquare(GIFRegColor, a-4, -0.25f, b-3, a-4, 0, b-3, a-4, 0, b-4, a-4, -0.25f, b-4);
				}

				if (a==7)
				{
					drawsquare(GIFRegColor, a-3, -0.25f, b-4, a-3, 0, b-4, a-3, 0, b-3, a-3, -0.25f, b-3);
				}

				if (b==0)
				{
					drawsquare(GIFRegColor, a-3, -0.25f, b-4, a-4, -0.25f, b-4, a-4, 0, b-4, a-3, 0, b-4);
				}

				if (b==7)
				{
					drawsquare(GIFRegColor, a-3, 0, b-3, a-4, 0, b-3, a-4, -0.25, b-3, a-3, -0.25, b-3);
				}

                                
				//here the board-drawing is finished and all nloops are done
				
				
				if (moveallowed(cursor2x,cursor2y,a,b)) //moveallowed
				{
				        
				
				        //if (squareisblack(a,b))
					  //glColor4f(0.8,0.8,0.8,1);
					//else
					  //glColor4f(0,0,0,1);
					
					matrixmanager.modelviewPushTranslate(a-3.5, 0, b-3.5);
					matrixmanager.modelviewPushRotate(1,0,0,270); //this makes the half-sphere face up
					
					if (squareisblack(a,b))
					  spherewhite->draw();  //draw half of sphere
					else
					  sphereblack->draw();  //draw half of sphere
					  
					matrixmanager.modelviewPop();
					matrixmanager.modelviewPop();
										
				}
				
			}
			
		}

		drawpieces();

}

template <class T>
void SPS2Chessboard<T>::relcursormove (int _relx,int _rely)
{
  if (posoffboard((cursorx + _relx),(cursory+_rely))) {return;}
  cursorx+=_relx;
  cursory+=_rely;

  if (!active)
  {
    //cursor 2 draggt mitherum
    cursor2x = cursorx;
    cursor2y = cursory;
  }
}

template <class T>
void SPS2Chessboard<T>::activate()
{
  active = (!active);

  if (active)
  {
    //cursor 2 bleibt picken
  }
  else
  {
    //eigentilcher Move erfolgt
    trymove(cursor2x,cursor2y,cursorx,cursory);

    //cursor draggt wieder mit herum
    cursor2x = cursorx;
    cursor2y = cursory;
  }
}

template<class T>
SPS2Chessboard<T>::~SPS2Chessboard()
{
  delete ringwhite;
  delete ringblack;
  
  delete spherewhite;
  delete sphereblack;
}


#endif
