//    Copyright (C) 2004 Michael Mahr

//    This file is part of "lesschess".
//
//    "lesschess" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "lesschess" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "lesschess"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "xascmodel.h"

//an ascmodel with additional postgenerated data: normals of all kind

XAscModel::XAscModel(std::string _filename, XAscMode mode) : AscModel(_filename)
{
  if ( ((mode & 1) == 1)  || ((mode & 2)==2))
  {
    //calculate facenormals
    for (std::map<unsigned int, AscFace>::const_iterator i = this->faces.begin(); i != this->faces.end(); i++)
    {
      //berechnen der Normalen: berechnen der Kantenvektoren: AB=v und AC=w  
      AscVector v;
      AscVector w;
    
      //AB
      v.x = this->verticles[i->second.b].x - this->verticles[i->second.a].x;
      v.y = this->verticles[i->second.b].y - this->verticles[i->second.a].y;
      v.z = this->verticles[i->second.b].z - this->verticles[i->second.a].z;    
  
      //AC
      w.x = this->verticles[i->second.c].x - this->verticles[i->second.a].x;
      w.y = this->verticles[i->second.c].y - this->verticles[i->second.a].y;
      w.z = this->verticles[i->second.c].z - this->verticles[i->second.a].z;    
 
      //berechnen der Facenormalen mit Hilfe der Determinante
      AscVector facenormal;
  
      facenormal.x = v.y * w.z - v.z * w.y;
      facenormal.y = v.z * w.x - v.x * w.z;
      facenormal.z = v.x * w.y - v.y * w.x; //<- there was the bug that cost me 2 hours
      
      //normalize the Facenormals - reduzieren auf Lnge 1
      float vectorlength = sqrt(pow(facenormal.x,2) + pow(facenormal.y,2) + pow(facenormal.z,2));
      
      facenormal.x /= vectorlength;
      facenormal.y /= vectorlength;
      facenormal.z /= vectorlength;

      facenormals[i->first] = facenormal;
    }
  }
  
  if ((mode & 2) == 2)
  {

    //calculate the vertex-normals by averaging the normals of the member faces
        
    for (std::map<unsigned int, AscVector>::const_iterator j = this->verticles.begin(); j != this->verticles.end(); j++)
    {
      unsigned int facecounter=0;
      AscVector vertexnormal;
      vertexnormal.x = 0;
      vertexnormal.y = 0;
      vertexnormal.z = 0;
        
      for(std::map<unsigned int, AscFace>::const_iterator k = this->faces.begin(); k != this->faces.end(); k++)
      {
        if ((j->first == k->second.a)||(j->first == k->second.b)||(j->first == k->second.c))
        {
      	  //if the face contains the verticle -> add the facenormal to the vertexnormal
          vertexnormal.x += facenormals[k->first].x;
          vertexnormal.y += facenormals[k->first].y;
          vertexnormal.z += facenormals[k->first].z;
  	  facecounter ++;
        }
      }


      //normalize    
      vertexnormal.x /= facecounter;
      vertexnormal.y /= facecounter;
      vertexnormal.z /= facecounter;
    
    
      vertexnormals[j->first] = vertexnormal;
    } 
  }//end if _vertexnormals    
}


XAscModel::~XAscModel()
{

}
